function [validPSF,valid_pointset] = ComputeValidPSFsNew2(I,pointset,psfSizeHalbe,background)
% Computes valid PSFs given positions of expected PSF centers and the image.
%
% input:   I - 3D image
%          pointeset - potential coordinates of PSFs
%          psfSizeHalbe - half the size of a PSF
%          background - background value
%
% output:  validPSF - struct containing the coordinates ('coord') and a set
% of PSFs ('psf')
%
% written by Maja Temerinac-Ott (C) 2011
% e-mail: temerina@informatik.uni-freiburg.de

I = double(I);
psfSize = 2*psfSizeHalbe + 1;
validPSF = struct('coord', {},'psf',{});
count = 1;

% Extract only the coordinates of the valid PSFs
for i = 1:size(pointset,2)
    coords = pointset(:,i);
    lx = round(coords(1)-psfSizeHalbe);
    if(lx<1)
        lx=1;
    end

    ux = round(coords(1)+psfSizeHalbe);
    if(ux>size(I,1))
        ux = size(I,1);
    end

    ly = round(coords(2)-psfSizeHalbe);
    if(ly<1)
        ly=1;
    end

    uy = round(coords(2)+psfSizeHalbe);
    if(uy>size(I,2))
        uy = size(I,2);
    end

    lz = round(coords(3)-psfSizeHalbe);
    if(lz<1)
        lz=1;
    end

    uz = round(coords(3)+psfSizeHalbe);
    if(uz>size(I,3))
        uz = size(I,3);
    end


    psf = I(lx:ux,ly:uy,lz:uz);
   
    
    psf = psf-background;
    psf(psf<0) = 0;
    psf = psf/sum(psf(:));
    
 
    
    %check whether no other psf is too close
    [neighborIds neighborDistances] = kNearestNeighbors(pointset', coords', 2);
    d = 0;
   
    dist = norm(coords-pointset(:,neighborIds(2)));
    if(dist<psfSize)
        d = 1;
    end

   
    if(sum(size(psf) - [psfSize,psfSize,psfSize]) == 0 && d==0)
        validPSF(count).coord = i;
        validPSF(count).psf = psf;
        count = count + 1;
    end

end

valid_pointset = zeros(3,count-1);

for i = 1:count-1
    k = validPSF(i).coord;
    valid_pointset(:,i) = pointset(:,k);
end
