% Test-script
% Performs deblurring of multiview (blurred+noisy) data using
% PSF extracted from the inserted beads 
%
% written by Maja Temerinac-Ott (C) 2011
% e-mail: temerina@informatik.uni-freiburg.de


clear all
close all


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read original and blurred images from the previously generated files
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dirname = 'test';
fname = 'rec_e7_dorsal_augeUINT8.h5';
I = hdf5read(fname,'/original');
I = double(I);
fname2 = 'rec_e7_dorsal_auge_psf.h5';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Extract Beads
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
angle = 90;
numImages = round(360/angle);
Images = cell(numImages,1);
detectedPointsets = cell(numImages,1);


t = 1000;
for i = 1:numImages
    I_noise = hdf5read(fname,['/angle',num2str((i-1)*angle)]);
    I_noise = double(I_noise);
    Images{i} = I_noise;
    I_psf = hdf5read(fname2,['/angle',num2str((i-1)*angle)]);
    I_psf = double(I_psf);
    [outim1,s1,pointset] = detectBeadsNew(I_psf,t);
     ImagePSFs{i} = I_psf;
    detectedPointsets{i} = pointset';
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Extract the PSF from the bead positions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
psfSize = 9;
background = 10;
psfSizeHalbe = floor(psfSize/2);
validPSFs = cell(numImages,1);
valid_pointsets = cell(numImages,1);
for i = 1:numImages
    [validPSF,valid_pointset] = ComputeValidPSFsNew2(ImagePSFs{i},detectedPointsets{i},psfSizeHalbe,background);
    validPSFs{i} = validPSF;
    valid_pointsets{i} = valid_pointset;
end 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Perform deconvolution blockwize
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fftBlockSize = 32;
N = fftBlockSize-2*psfSize;

ImagesPadded = cell(numImages,1);
for i = 1:numImages
    ImagesPadded{i} = padarray(Images{i}, [psfSize,psfSize,psfSize]);
end


ModelInit = Images{1};
for i = 2:numImages
    ModelInit = ModelInit + Images{i};
end
ModelInit = ModelInit/numImages;%Average of the noisy images
display('starting iteration')
totalIter = 20;
lambda =0.002;
wname = 'Multiview4_eye_e7_UINT8_Iter';
[Iresult,snr] = computeBlockwizeDeconvolutionMultiview_alternate(ModelInit,I,ImagesPadded,validPSFs,valid_pointsets,psfSize,N,totalIter,dirname,wname,lambda);

%Show SNR
save('snr_eye_e7.mat', 'snr')
figure
plot(snr);
xlabel('Iteration')
ylabel('SNR')

%show central slice in xy and xz
sliceNum = 62;
offset = 10;
num_iter = 10;
Ires = hdf5read([dirname,wname,num2str(num_iter),'.h5'],'/data');
Ires = uint8(Ires);

I_noise=uint8(I_noise);
A_rec = I_noise(offset:end-offset-1,offset:end-offset-1,sliceNum);
B_rec = squeeze(I_noise(sliceNum,offset:end-offset-1,offset:end-offset-1));
imwrite(A_rec,'eye_e7_xy_recorded.png')
imwrite(B_rec,'eye_e7_xz_recorded.png')

A = Ires(offset:end-offset-1,offset:end-offset-1,sliceNum);
B = squeeze(Ires(sliceNum,offset:end-offset-1,offset:end-offset-1));
imwrite(A,['eye_e7_xy_lambda',num2str(lambda), '.png'])
imwrite(B,['eye_e7_xz_lambda',num2str(lambda),'.png'])

figure;
subplot(2,2,1), imshow(A_rec);title('xy-slice, z=62, recorded')
subplot(2,2,2), imshow(B_rec);title('xz-slice, y=62, recorded')
subplot(2,2,3), imshow(A);title(['xy-slice, z=62, iterations=', num2str(num_iter), ', lambda=', num2str(lambda)])
subplot(2,2,4), imshow(B);title(['xz-slice, y=62, iterations=', num2str(num_iter), ', lambda=', num2str(lambda)])

