function [Iresult, snr] = computeBlockwizeDeconvolutionMultiview_alternate(ModelInit, I, ImagesPadded, validPSFs, valid_pointsets, psfSize, N, totalIter,dirname,fname,lambda)
% Performs blockwize deblurring for set of 3D Images and a set of corresponding PSFs associated with each image. The groundtruth is available, so the SNR can be computed in each iteration step. 
%
% input:   ModelInit - initial 3D image
%          I - ground truth 3D image
%          validPSFs - cell array containing valid PSFs for each image
%          valid_pointsets - cell array containing valid 3D coordinates of
%          the PSFs in each image
%          psfSize - M size of the PSF
%          N - size of the NxNxN block
%          totalIter - number of iterations
%          dirname - path for writing the output files
%          fname - filename of the outputfile
%          lambda - regularization parameter
%
% output:  Iresult - deblurred 3D image
%          snr -  array with SNR for each iteration step
%
% written by Maja Temerinac-Ott (C) 2011
% e-mail: temerina@informatik.uni-freiburg.de

Iresult = ModelInit;
Iresult_pad = padarray(Iresult, [psfSize,psfSize,psfSize]);

numImages = length(ImagesPadded);
imgBlock = cell(numImages,1);
psfBlock = cell(numImages,1);

snr = zeros(1,totalIter);

start =N/2;
for numIter = 1:totalIter
        
    if mod(numIter,2) == 0
        start =N/2;
    end
    
    if mod(numIter,2) == 1
        start = N;
    end
        
         tic 
         for k1=start:N:size(I,1)-N
             for k2=start:N:size(I,2)-N
                 for k3=start:N:size(I,3)-N
                     for i = 1:numImages
                        imgBlock{i} = ImagesPadded{i}(k1:k1+N-1+2*psfSize,k2:k2+N-1+2*psfSize,k3:k3+N-1+2*psfSize);
                        psfBlock{i} = estimatePSFsNew2(validPSFs{i},valid_pointsets{i},k1+N/2+psfSize,k2+N/2+psfSize,k3+N/2+psfSize,psfSize);
                     end
                    Model = Iresult_pad(k1:k1+N-1+2*psfSize,k2:k2+N-1+2*psfSize,k3:k3+N-1+2*psfSize);
                    deconvolved = DeconvolveLucyRichardson_3D_multiview(imgBlock,psfBlock,1,Model,lambda);
                    
               
                   
                    Iresult(k1:k1+N-1,k2:k2+N-1,k3:k3+N-1) = deconvolved(psfSize+1:end-psfSize,psfSize+1:end-psfSize,psfSize+1:end-psfSize);                   
                 end
             end
            
         end
         toc
  
    Iresult(Iresult>2^8-1) = 2^8-1;
    Iresult(Iresult<0) = 0;

    if(mod(numIter,5) == 0)
        hdf5write([dirname,fname,num2str(numIter),'.h5'],'data',Iresult); 
        display([dirname,fname,num2str(numIter),'.h5', '  written'])
    end
    
    Iresult_pad = padarray(Iresult, [psfSize,psfSize,psfSize]);
    display(['numIter = ',num2str(numIter) ])
    val2 = computeSNR(double(I),Iresult);
    display(['SNR after Recontruction: ',num2str(val2)])
    snr(numIter) = val2;
    
 end
