function psf = estimatePSFsNew2(validPSF,valid_pointset,k1,k2,k3,psfSize)
% Interpolates a PSF from a set of valid PSFs.
%
% input:   validPSF - struct with valid PSFs
%          valid_pointeset - Nx3 array with valid PSF coordinates
%          k1 - x-coordinate
%          k2 - y-coordinate
%          k3 - z-coordinate
%          psfSize - M size of the PSF
%
% output:  psf - MxMxM array 
%
% written by Maja Temerinac-Ott (C) 2011
% e-mail: temerina@informatik.uni-freiburg.de

psf = zeros(psfSize,psfSize,psfSize);

% search only in the valid pointset
xcoord = k1*ones(1,size(valid_pointset,2));
ycoord = k2*ones(1,size(valid_pointset,2));
zcoord = k3*ones(1,size(valid_pointset,2));
dist = sum(([xcoord; ycoord;zcoord] - valid_pointset).^2);


sigma = 1;
weightTotal = 0;
for i = 1:size(valid_pointset,2)
    weight = exp(-(sqrt(dist(i))/(2*sigma*sigma)));
    psf = psf + weight*validPSF(i).psf;
    weightTotal =  weightTotal + weight;
end

psf = psf/weightTotal;



          
